-- Licensed to the Apache Software Foundation (ASF) under one
-- or more contributor license agreements.  See the NOTICE file
-- distributed with this work for additional information
-- regarding copyright ownership.  The ASF licenses this file
-- to you under the Apache License, Version 2.0 (the
-- "License"); you may not use this file except in compliance
-- with the License.  You may obtain a copy of the License at
--
--   http://www.apache.org/licenses/LICENSE-2.0
--
-- Unless required by applicable law or agreed to in writing,
-- software distributed under the License is distributed on an
-- "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
-- KIND, either express or implied.  See the License for the
-- specific language governing permissions and limitations
-- under the License.
--
-- --------------------------------------------------------------------
--
-- cdb_schema.sql
--
-- Define mpp administrative schema and several SQL functions to aid 
-- in maintaining the mpp administrative schema.  
--
-- This is version 2 of the schema.
--
-- TODO Error checking is rudimentary and needs improvment.
--
--
-- --------------------------------------------------------------------
SET log_min_messages = WARNING;

-------------------------------------------------------------------
-- database
-------------------------------------------------------------------
CREATE OR REPLACE VIEW gp_pgdatabase AS 
    SELECT *
      FROM gp_pgdatabase() AS L(dbid smallint, isprimary boolean, content smallint, valid boolean, definedprimary boolean);

GRANT SELECT ON gp_pgdatabase TO PUBLIC;

------------------------------------------------------------------
-- distributed transaction related
------------------------------------------------------------------
CREATE OR REPLACE VIEW gp_distributed_xacts AS 
    SELECT *
      FROM gp_distributed_xacts() AS L(distributed_xid xid, distributed_id text, state text, gp_session_id int, xmin_distributed_snapshot xid);

GRANT SELECT ON gp_distributed_xacts TO PUBLIC;


CREATE OR REPLACE VIEW gp_transaction_log AS 
    SELECT *
      FROM gp_transaction_log() AS L(segment_id smallint, dbid smallint, transaction xid, status text);

GRANT SELECT ON gp_transaction_log TO PUBLIC;

CREATE OR REPLACE VIEW gp_distributed_log AS 
    SELECT *
      FROM gp_distributed_log() AS L(segment_id smallint, dbid smallint, distributed_xid xid, distributed_id text, status text, local_transaction xid);

GRANT SELECT ON gp_distributed_log TO PUBLIC;


------------------------------------------------------------------
-- plpgsql is created by default in 3.4, but it is still a "user" language,
-- rather than an "internal" language.
------------------------------------------------------------------
CREATE LANGUAGE plpgsql;

--ALTER RESOURCE QUEUE pg_default WITH (priority=medium, memory_limit='-1');
--ALTER RESOURCE QUEUE pg_default WITH (memory_limit='-1');
CREATE SCHEMA hawq_toolkit;

RESET log_min_messages;
-- --------------------------------------------------------------------
--
-- cdb_external_extensions.sql
--
-- External Extensions, including custom formatter, protocol
--
--
-- --------------------------------------------------------------------
  
------------------------------------------------------------------
-- pxf
------------------------------------------------------------------

CREATE OR REPLACE FUNCTION pg_catalog.pxf_write() RETURNS integer
AS '$libdir/pxf.so', 'pxfprotocol_export'
LANGUAGE C STABLE;

CREATE OR REPLACE FUNCTION pg_catalog.pxf_read() RETURNS integer
AS '$libdir/pxf.so', 'pxfprotocol_import'
LANGUAGE C STABLE;

CREATE OR REPLACE FUNCTION pg_catalog.pxf_validate() RETURNS void
AS '$libdir/pxf.so', 'pxfprotocol_validate_urls'
LANGUAGE C STABLE;

CREATE OR REPLACE FUNCTION pg_catalog.pxfwritable_export(record) RETURNS bytea
AS '$libdir/pxf.so', 'gpdbwritableformatter_export'
LANGUAGE C STABLE;

CREATE OR REPLACE FUNCTION pg_catalog.pxfwritable_import() RETURNS record
AS '$libdir/pxf.so', 'gpdbwritableformatter_import'
LANGUAGE C STABLE;

CREATE TRUSTED PROTOCOL pxf (
  writefunc		= pxf_write,
  readfunc      = pxf_read,
  validatorfunc = pxf_validate);  
  
------------------------------------------------------------------
-- fixedwidth Formatters
------------------------------------------------------------------
  
CREATE OR REPLACE FUNCTION fixedwidth_in() RETURNS record 
AS '$libdir/fixedwidth.so', 'fixedwidth_in'
LANGUAGE C STABLE;

CREATE OR REPLACE FUNCTION fixedwidth_out(record) RETURNS bytea 
AS '$libdir/fixedwidth.so', 'fixedwidth_out'
LANGUAGE C STABLE;

------------------------------------------------------------------
-- external HDFS
------------------------------------------------------------------
CREATE OR REPLACE FUNCTION hdfs_validate() RETURNS void
AS '$libdir/exthdfs.so', 'hdfsprotocol_validate'
LANGUAGE C STABLE;

CREATE OR REPLACE FUNCTION hdfs_blocklocation() RETURNS void
AS '$libdir/exthdfs.so', 'hdfsprotocol_blocklocation'
LANGUAGE C STABLE;

------------------------------------------------------------------
-- external HIVE
------------------------------------------------------------------
CREATE OR REPLACE FUNCTION hive_validate() RETURNS void
AS '$libdir/exthive.so', 'hiveprotocol_validate'
LANGUAGE C STABLE;

CREATE OR REPLACE FUNCTION hive_blocklocation() RETURNS void
AS '$libdir/exthive.so', 'hiveprotocol_blocklocation'
LANGUAGE C STABLE;

------------------------------------------------------------------
-- csv Formatters
------------------------------------------------------------------
  
CREATE OR REPLACE FUNCTION csv_in() RETURNS record 
AS '$libdir/extfmtcsv.so', 'extfmtcsv_in'
LANGUAGE C STABLE;

CREATE OR REPLACE FUNCTION csv_out(record) RETURNS bytea 
AS '$libdir/extfmtcsv.so', 'extfmtcsv_out'
LANGUAGE C STABLE;

------------------------------------------------------------------
-- text Formatters
------------------------------------------------------------------
  
CREATE OR REPLACE FUNCTION text_in() RETURNS record 
AS '$libdir/extfmtcsv.so', 'extfmttext_in'
LANGUAGE C STABLE;

CREATE OR REPLACE FUNCTION text_out(record) RETURNS bytea 
AS '$libdir/extfmtcsv.so', 'extfmttext_out'
LANGUAGE C STABLE;-- --------------------------------------------------------------------
--
-- filesystem.sql
--
-- User defined filesystems
--
--
-- --------------------------------------------------------------------

------------------------------------------------------------------
-- Create HDFS filesystem
------------------------------------------------------------------

CREATE FILESYSTEM HDFS
(
    gpfs_libfile = "$libdir/gpfshdfs.so",
    gpfs_connect = "gpfs_hdfs_connect",
    gpfs_disconnect = "gpfs_hdfs_disconnect",
    gpfs_open = "gpfs_hdfs_openfile",
    gpfs_close = "gpfs_hdfs_closefile",
    gpfs_seek = "gpfs_hdfs_seek",
    gpfs_tell = "gpfs_hdfs_tell",
    gpfs_read = "gpfs_hdfs_read",
    gpfs_write = "gpfs_hdfs_write",
    gpfs_flush = "gpfs_hdfs_sync",
    gpfs_delete = "gpfs_hdfs_delete",
    gpfs_chmod = "gpfs_hdfs_chmod",
    gpfs_mkdir = "gpfs_hdfs_createdirectory",
    gpfs_truncate = "gpfs_hdfs_truncate",
    gpfs_getpathinfo = "gpfs_hdfs_getpathinfo",
    gpfs_freefileinfo = "gpfs_hdfs_freefileinfo"
);

-- --------------------------------------------------------------------
--
-- orc_init.sql
--
-- Support ORC format in pluggable storage framework at initialization
--
-- --------------------------------------------------------------------
  
CREATE OR REPLACE FUNCTION pg_catalog.orc_validate_interfaces() RETURNS void
AS '$libdir/orc.so', 'orc_validate_interfaces'
LANGUAGE C STABLE;

CREATE OR REPLACE FUNCTION pg_catalog.orc_validate_options() RETURNS void
AS '$libdir/orc.so', 'orc_validate_options'
LANGUAGE C STABLE;

CREATE OR REPLACE FUNCTION pg_catalog.orc_validate_encodings() RETURNS void
AS '$libdir/orc.so', 'orc_validate_encodings'
LANGUAGE C STABLE;

CREATE OR REPLACE FUNCTION pg_catalog.orc_validate_datatypes() RETURNS void
AS '$libdir/orc.so', 'orc_validate_datatypes'
LANGUAGE C STABLE;

CREATE OR REPLACE FUNCTION pg_catalog.orc_beginscan() RETURNS bytea
AS '$libdir/orc.so', 'orc_beginscan'
LANGUAGE C STABLE;

CREATE OR REPLACE FUNCTION pg_catalog.orc_getnext_init() RETURNS bytea
AS '$libdir/orc.so', 'orc_getnext_init'
LANGUAGE C STABLE;

CREATE OR REPLACE FUNCTION pg_catalog.orc_getnext() RETURNS bytea
AS '$libdir/orc.so', 'orc_getnext'
LANGUAGE C STABLE;

CREATE OR REPLACE FUNCTION pg_catalog.orc_rescan() RETURNS void
AS '$libdir/orc.so', 'orc_rescan'
LANGUAGE C STABLE;

CREATE OR REPLACE FUNCTION pg_catalog.orc_endscan() RETURNS void
AS '$libdir/orc.so', 'orc_endscan'
LANGUAGE C STABLE;

CREATE OR REPLACE FUNCTION pg_catalog.orc_stopscan() RETURNS void
AS '$libdir/orc.so', 'orc_stopscan'
LANGUAGE C STABLE;

CREATE OR REPLACE FUNCTION pg_catalog.orc_insert_init() RETURNS bytea
AS '$libdir/orc.so', 'orc_insert_init'
LANGUAGE C STABLE;

CREATE OR REPLACE FUNCTION pg_catalog.orc_insert() RETURNS bytea
AS '$libdir/orc.so', 'orc_insert'
LANGUAGE C STABLE;

CREATE OR REPLACE FUNCTION pg_catalog.orc_insert_finish() RETURNS void
AS '$libdir/orc.so', 'orc_insert_finish'
LANGUAGE C STABLE;

